(*---------------------------------------------------------------------------*
  INTERFACE  cf_scmonad.mli

  Copyright (c) 2002-2006, James H. Woodyatt
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGE. 
 *---------------------------------------------------------------------------*)

(** The state-continuation monad and its operators. *)

(** {6 Overview}
    
    The state-continuation monad is provided here purely for reference
    purposes.  It may be helpful as an example of how to lift the continuation
    monad into more complex monads.
*)

(** {6 Types} *)

(** The state-continuation monad. *)
type ('s, 'x, 'a) t = ('s -> 'x, 'a) Cf_cmonad.t

(** {6 Modules} *)

(** The continuation monad: a function for passing intermediate results from
    continuation context to continuation context with an encapsulated state
    value at each stage.
*)
module Op: sig
    (** Use [m >>= f] to produce a monad that applies [f] to the result of
        evaluating [m].
    *)
    val ( >>= ): ('s, 'x, 'a) t -> ('a -> ('s, 'x, 'b) t) -> ('s, 'x, 'b) t
end

(** {6 Operators} *)

(** A monad that returns [unit] and performs no operation. *)
val nil: ('s, 'x, unit) t

(** Use [return a] to produce a monad that returns [a]. *)
val return: 'a -> ('s, 'x, 'a) t

(** Use [init x] to produce a monad that discards the current intermediate
    context and passes [x] into the continuation.
*)
val init: 'x -> ('s, 'x, 'a) t

(** Use [cont f] to produce a monad that applies [f] to the current
	intermediate context and passes the result into the continuation.
*)
val cont: ('x -> 'x) -> ('s, 'x, unit) t

(** A monad that returns the encapsulate state. *)
val load: ('s, 'x, 's) t

(** Use [store s] to produce a monad with [s] as the encapsulated state. *)
val store: 's -> ('s, 'x, unit) t

(** Use [modify f] to produce a monad that applies [f] to the encapsulated
    state to obtain a new state value.
*)
val modify: ('s -> 's) -> ('s, 'x, unit) t

(** Use [field f] to produce a monad that applies [f] to the encapsulated state
	and returns the result.
*)
val field: ('s -> 'a) -> ('s, 'x, 'a) t

(** Use [down m s] to produce a stateless continuation monad from a
    state-continuation monad [m] and an initial state [s].
*)
val down: ('s, 'x, unit) t -> 's -> ('x, 's) Cf_cmonad.t

(** Use [liftC m] to lift a stateless continuation monad [m] into a
    state-continuation monad.
*)
val liftC: ('x, 'a) Cf_cmonad.t -> ('s, 'x, 'a) t

(** Use [liftS m] to lift a state monad [m] into a state-continuation monad. *)
val liftS: ('s, 'a) Cf_smonad.t -> ('s, 'x, 'a) t

(** Use [eval m s] to evaluate the state-continuation monad [m] with initial
	state [s] to produce a function from an initial context to a final context.
*)
val eval: ('s, 'x, unit) t -> 's -> 'x -> 'x

(** Use [bridge x f m] to lower the state-continuation monad [m], then bridge
	with the initial context [x] and a map function [f], and finally lift the
	result to another state-contination monad, one that evaluates [m] using the
	state bridged from the result monad.
*)
val bridge: 'x -> ('x -> 'y) -> ('s, 'x, unit) t -> ('s, 'y, unit) t

(*--- End of File [ cf_scmonad.mli ] ---*)
